@@ -23,7 +23,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/afs/LINUX/osi_vnodeops.c,v 1.29.2.17 2002/08/01 19:12:01 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/LINUX/osi_vnodeops.c,v 1.29.2.25 2004/10/21 15:01:20 shadow Exp $");
 
 #include "../afs/sysincludes.h"
 #include "../afs/afsincludes.h"
@@ -540,6 +540,16 @@ static int afs_linux_lock(struct file *fp, int cmd, struct file_lock *flp)
     flock.l_start = flp->fl_start;
     flock.l_len = flp->fl_end - flp->fl_start;
 
+    /* Safe because there are no large files, yet */
+#if defined(F_GETLK64) && (F_GETLK != F_GETLK64)
+    if (cmd == F_GETLK64)
+	cmd = F_GETLK;
+    else if (cmd == F_SETLK64)
+	cmd = F_SETLK;
+    else if (cmd == F_SETLKW64)
+	cmd = F_SETLKW;
+#endif /* F_GETLK64 && F_GETLK != F_GETLK64 */
+
     AFS_GLOCK();
     code = afs_lockctl(vcp, &flock, cmd, credp);
     AFS_GUNLOCK();
@@ -841,13 +851,14 @@ static int afs_linux_dentry_revalidate(struct dentry *dp)
 /* afs_dentry_iput */
 static void afs_dentry_iput(struct dentry *dp, struct inode *ip)
 {
+    int isglock;
     if (ICL_SETACTIVE(afs_iclSetp)) {
-	AFS_GLOCK();
+	if (!isglock) AFS_GLOCK();
 	afs_Trace3(afs_iclSetp, CM_TRACE_DENTRYIPUT,
 		   ICL_TYPE_POINTER, ip,
 		   ICL_TYPE_STRING, dp->d_parent->d_name.name,
 		   ICL_TYPE_STRING, dp->d_name.name);
-	AFS_GUNLOCK();
+	if (!isglock) AFS_GUNLOCK();
     }
 
     osi_iput(ip);
@@ -855,12 +866,13 @@ static void afs_dentry_iput(struct dentry *dp, struct inode *ip)
 
 static int afs_dentry_delete(struct dentry *dp)
 {
+    int isglock;
     if (ICL_SETACTIVE(afs_iclSetp)) {
-	AFS_GLOCK();
+	if (!isglock) AFS_GLOCK();
 	afs_Trace3(afs_iclSetp, CM_TRACE_DENTRYDELETE, ICL_TYPE_POINTER, 
 		   dp->d_inode, ICL_TYPE_STRING, dp->d_parent->d_name.name,
 		   ICL_TYPE_STRING, dp->d_name.name);
-	AFS_GUNLOCK();
+	if (!isglock) AFS_GUNLOCK();
     }
 
     if (dp->d_inode && (ITOAFS(dp->d_inode)->states & CUnlinked))
@@ -1320,6 +1332,12 @@ int afs_linux_writepage(struct page *pp)
     unsigned offset = PAGE_CACHE_SIZE;
     long status;
  
+#ifdef PageLaunder
+    if (PageLaunder(pp)) {
+        return(fail_writepage(pp));
+    }
+#endif
+
     inode = (struct inode *) mapping->host;
     end_index = inode->i_size >> PAGE_CACHE_SHIFT;
 
