@@ -22,7 +22,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_remove.c,v 1.4.4.8 2002/06/08 00:13:41 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/afs/VNOPS/afs_vnop_remove.c,v 1.4.4.11 2004/10/18 05:45:40 shadow Exp $");
 
 #include "../afs/sysincludes.h"	/* Standard vendor system headers */
 #include "../afs/afsincludes.h"	/* Afs-based standard headers */
@@ -226,6 +226,7 @@ afs_remove(OSI_VC_ARG(adp), aname, acred)
     afs_int32 offset, len;
     struct AFSFetchStatus OutDirStatus;
     struct AFSVolSync tsync;
+    struct afs_fakestat_state fakestate;
     XSTATS_DECLS
     OSI_VC_CONVERT(adp)
 
@@ -237,15 +238,6 @@ afs_remove(OSI_VC_ARG(adp), aname, acred)
     tvc = (struct vcache *)ndp->ni_vp;  /* should never be null */
 #endif
 
-    /* Check if this is dynroot */
-    if (afs_IsDynroot(adp)) {
-#ifdef  AFS_OSF_ENV
-        afs_PutVCache(adp, 0);
-        afs_PutVCache(tvc, 0);
-#endif
-	return afs_DynrootVOPRemove(adp, acred, aname);
-    }
-
     if (code = afs_InitReq(&treq, acred)) {
 #ifdef  AFS_OSF_ENV
         afs_PutVCache(adp, 0);
@@ -253,7 +245,30 @@ afs_remove(OSI_VC_ARG(adp), aname, acred)
 #endif
       return code;
     }
+
+    afs_InitFakeStat(&fakestate);
+    code = afs_EvalFakeStat(&adp, &fakestate, &treq);
+    if (code) {
+	afs_PutFakeStat(&fakestate);
+#ifdef  AFS_OSF_ENV
+	afs_PutVCache(adp, 0);
+	afs_PutVCache(tvc, 0);
+#endif
+	return code;
+    }
+
+    /* Check if this is dynroot */
+    if (afs_IsDynroot(adp)) {
+	code = afs_DynrootVOPRemove(adp, acred, aname);
+	afs_PutFakeStat(&fakestate);
+#ifdef  AFS_OSF_ENV
+	afs_PutVCache(adp, 0);
+	afs_PutVCache(tvc, 0);
+#endif
+	return code;
+    }
     if (strlen(aname) > AFSNAMEMAX) {
+	afs_PutFakeStat(&fakestate);
 #ifdef  AFS_OSF_ENV
 	afs_PutVCache(adp, 0);
 	afs_PutVCache(tvc, 0);
@@ -267,13 +282,15 @@ tagain:
     if (code) {
 	afs_PutVCache(adp, 0);
 	afs_PutVCache(tvc, 0);
+	afs_PutFakeStat(&fakestate);
 	return afs_CheckCode(code, &treq, 22);
     }
 #else	/* AFS_OSF_ENV */
     tvc = (struct vcache *) 0;
     if (code) {
-      code = afs_CheckCode(code, &treq, 23);
-      return code;
+	code = afs_CheckCode(code, &treq, 23);
+	afs_PutFakeStat(&fakestate);
+	return code;
     }
 #endif
 
@@ -286,6 +303,7 @@ tagain:
         afs_PutVCache(tvc, 0);
 #endif
         code = EROFS;
+	afs_PutFakeStat(&fakestate);
 	return code;
     }
 
@@ -349,10 +367,18 @@ tagain:
     Tadp1 = adp; Tadpr = VREFCOUNT(adp); Ttvc = tvc; Tnam = aname; Tnam1 = 0;
     if (tvc) Ttvcr = VREFCOUNT(tvc);
 #ifdef	AFS_AIX_ENV
-    if (tvc && (VREFCOUNT(tvc) > 2) && tvc->opens > 0 && !(tvc->states & CUnlinked)) {
+    if (tvc && (VREFCOUNT(tvc) > 2) && tvc->opens > 0
+	&& !(tvc->states & CUnlinked)) 
 #else
-    if (tvc && (VREFCOUNT(tvc) > 1) && tvc->opens > 0 && !(tvc->states & CUnlinked)) {
+#ifdef AFS_DARWIN14_ENV
+      if (tvc && (VREFCOUNT(tvc) > 1 + DARWIN_REFBASE) && tvc->opens > 0 && !(tvc->states & CUnlinked)) 
+#else
+      if (tvc && (VREFCOUNT(tvc) > 1) && tvc->opens > 0
+	  && !(tvc->states & CUnlinked)) 
 #endif
+#endif
+      {
+
 	char *unlname = newname();
 
 	ReleaseWriteLock(&adp->lock);
@@ -378,6 +404,7 @@ tagain:
 #ifdef	AFS_OSF_ENV
     afs_PutVCache(adp, WRITE_LOCK);
 #endif	/* AFS_OSF_ENV */
+    afs_PutFakeStat(&fakestate);
     return code;
 }
 
@@ -398,6 +425,7 @@ afs_remunlink(avc, doit)
     struct VenusFid dirFid;
     register struct dcache *tdc;
     afs_int32 offset, len, code=0;
+    int oldref;
 
     if (NBObtainWriteLock(&avc->lock, 423))
 	return 0;
@@ -415,7 +443,7 @@ afs_remunlink(avc, doit)
 	    cred = avc->uncred;
 	    avc->uncred = NULL;
 
-#ifdef AFS_DARWIN_ENV
+#if defined(AFS_DARWIN_ENV) && !defined(AFS_DARWIN14_ENV)
            /* this is called by vrele (via VOP_INACTIVE) when the refcount
               is 0. we can't just call VN_HOLD since vref will panic.
               we can't just call osi_vnhold because a later AFS_RELE will call
@@ -433,6 +461,12 @@ afs_remunlink(avc, doit)
 	     */
 	    avc->states  &= ~(CUnlinked | CUnlinkedDel);
 
+#ifdef AFS_DARWIN14_ENV
+	    if (VREFCOUNT(avc) < 4) {
+		oldref = 4 - VREFCOUNT(avc);
+		VREFCOUNT_SET(avc, 4);
+	    }
+#endif
 	    ReleaseWriteLock(&avc->lock);
 
 	    dirFid.Cell = avc->fid.Cell;
@@ -456,8 +490,15 @@ afs_remunlink(avc, doit)
 	    osi_FreeSmallSpace(unlname);
 	    crfree(cred);
 #ifdef AFS_DARWIN_ENV
+#ifndef AFS_DARWIN14_ENV
 	    osi_Assert(VREFCOUNT(avc) == 1);
 	    VREFCOUNT_SET(avc, 0);
+#else
+	    if (oldref) {
+		int newref = VREFCOUNT(avc) - oldref;
+		VREFCOUNT_SET(avc, newref);
+	    }
+#endif
 #endif
         }
     }
