@@ -82,7 +82,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/viced/callback.c,v 1.6.2.8 2003/03/11 02:22:03 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/viced/callback.c,v 1.6.2.11 2004/10/20 07:41:29 shadow Exp $");
 
 #include <stdio.h> 
 #include <stdlib.h>      /* for malloc() */
@@ -442,7 +442,7 @@ static CDel(cb)
 	assert(0);
 	ViceLog(0,("CDel: Internal Error -- shutting down: wanted %d from %d, now at %d\n",cbi,fe->firstcb,*cbp));
 	DumpCallBackState();
-	ShutDown();
+	ShutDownAndCore(PANIC);
       }
     }
     CDelPtr(fe, cbp);
@@ -492,7 +492,7 @@ static u_short *FindCBPtr(fe, host)
 	if (safety > cbstuff.nblks) {
 	  ViceLog(0,("FindCBPtr: Internal Error -- shutting down.\n"));
 	  DumpCallBackState();
-	  ShutDown();
+	  ShutDownAndCore(PANIC);
 	}
 	cb = itocb(*cbp);
 	if (cb->hhead == hostindex)
@@ -695,7 +695,7 @@ AddCallBack1_r(host, fid, thead, type, locked)
 	if (safety > cbstuff.nblks) {
 	  ViceLog(0,("AddCallBack1: Internal Error -- shutting down.\n"));
 	  DumpCallBackState();
-	  ShutDown();
+	  ShutDownAndCore(PANIC);
 	}
 	if (cb->hhead == h_htoi(host))
 	    break;
@@ -772,6 +772,7 @@ static void MultiBreakCallBack_r(cba, ncbas, afidp, xhost)
     if (!thishost || (thishost->hostFlags & HOSTDELETED)) {
       continue;
     }
+    rx_GetConnection(thishost->callback_rxcon);
     conns[j++] = thishost->callback_rxcon;
 	
 #ifdef	ADAPT_MTU
@@ -842,8 +843,10 @@ static void MultiBreakCallBack_r(cba, ncbas, afidp, xhost)
   for (i=0; i<ncbas; i++) {
     struct host *hp;
     hp = cba[i].hp;
-    if (hp && xhost != hp)
+    if (hp && xhost != hp) {
+      rx_PutConnection(hp->callback_rxcon);
       h_Release_r(hp);
+    }
   }
 
 return ;
@@ -1361,7 +1364,7 @@ CleanupTimedOutCallBacks_r()
 		if (ntimedout > cbstuff.nblks) {
 		  ViceLog(0,("CCB: Internal Error -- shutting down...\n"));
 		  DumpCallBackState();
-		  ShutDown();
+		  ShutDownAndCore(PANIC);
 		}
 	    } while (cbi != *thead);
 	    *thead = 0;
@@ -1376,17 +1379,23 @@ CleanupTimedOutCallBacks_r()
 
 
 static struct host *lih_host;
+static int lih_host_held = 0;
 
 static int lih_r(host, held, hostp)
     register struct host *host, *hostp;
     register int held;
 
 {
+    lih_host_held = 0;
     if (host->cblist
 	   && ((hostp && host != hostp) || (!held && !h_OtherHolds_r(host)))
            && (!lih_host || host->ActiveCall < lih_host->ActiveCall) ) {
 	lih_host = host;
     }
+    if (!held) {
+        held = 1;
+        lih_host_held = 1;
+    }
     return held;
 
 } /*lih*/
@@ -1415,8 +1424,13 @@ static int GetSomeSpace_r(hostp, locked)
 	hp = lih_host;
 	if (hp) {
 	    cbstuff.GSS4++;
-	    if ( ! ClearHostCallbacks_r(hp, 0 /* not locked or held */) )
-		return;
+            if (!ClearHostCallbacks_r(hp, 0 /* not locked or held */ )) {
+                if (lih_host_held) 
+                    h_Release_r(hp);
+                return 0;
+            }
+            if (lih_host_held) 
+                h_Release_r(hp);
 	    hp2 = hp->next;
 	} else {
 	    hp2 = hostList;
